@@ -34,7 +34,7 @@ from craft_cli import (
 from craft_store import errors
 
 from charmcraft import config, __version__, env, utils
-from charmcraft.commands import build, clean, init, pack, store, version, analyze
+from charmcraft.commands import clean, init, pack, store, version, analyze
 from charmcraft.commands.store.client import ALTERNATE_AUTH_ENV_VAR
 from charmcraft.parts import setup_parts
 
@@ -60,7 +60,6 @@ See https://charmhub.io/publishing for more information.
 # important when listing commands and showing help.
 _basic_commands = [
     analyze.AnalyzeCommand,
-    build.BuildCommand,
     clean.CleanCommand,
     pack.PackCommand,
     init.InitCommand,
@@ -116,6 +115,20 @@ def _get_system_details():
     return f"System details: {os_platform}; Environment: {env_string}"
 
 
+def _emit_error(error, cause=None):
+    """Emit the error in a centralized way so we can alter it consistently."""
+    # set the cause, if any
+    if cause is not None:
+        error.__cause__ = cause
+
+    # if it's a charmcraft running inside a provided instance, do not report the internal logpath
+    if env.is_charmcraft_running_in_managed_mode():
+        error.logpath_report = False
+
+    # finally, emit
+    emit.error(error)
+
+
 def main(argv=None):
     """Provide the main entry point."""
     if env.is_charmcraft_running_in_managed_mode():
@@ -124,7 +137,7 @@ def main(argv=None):
         logpath = None
 
     emit.init(
-        EmitterMode.NORMAL,
+        EmitterMode.BRIEF,
         "charmcraft",
         f"Starting charmcraft version {__version__}",
         log_filepath=logpath,
@@ -163,7 +176,7 @@ def main(argv=None):
                 "the current directory or where specified with --project-dir option); see "
                 "the reference: https://discourse.charmhub.io/t/charmcraft-configuration/4138"
             )
-        emit.trace(_get_system_details())
+        emit.debug(_get_system_details())
         retcode = dispatcher.run()
 
     except ArgumentParsingError as err:
@@ -175,21 +188,19 @@ def main(argv=None):
         emit.ended_ok()
         retcode = 0
     except CraftError as err:
-        emit.error(err)
+        _emit_error(err)
         retcode = err.retcode
     except errors.CraftStoreError as err:
         error = CraftError(f"craft-store error: {err}")
-        emit.error(error)
+        _emit_error(error)
         retcode = 1
     except KeyboardInterrupt as exc:
         error = CraftError("Interrupted.")
-        error.__cause__ = exc
-        emit.error(error)
+        _emit_error(error, cause=exc)
         retcode = 1
     except Exception as err:
         error = CraftError(f"charmcraft internal error: {err!r}")
-        error.__cause__ = err
-        emit.error(error)
+        _emit_error(error, cause=err)
         retcode = 1
     else:
         emit.ended_ok()
