@@ -16,13 +16,16 @@
 
 """Collection of utilities for charmcraft."""
 
+import datetime
 import os
 import pathlib
 import platform
 import sys
+import time
 from collections import namedtuple
 from dataclasses import dataclass
 from stat import S_IRGRP, S_IROTH, S_IRUSR, S_IXGRP, S_IXOTH, S_IXUSR
+from typing import Iterable
 
 import yaml
 from craft_cli import emit, CraftError
@@ -62,13 +65,13 @@ def make_executable(fh):
 def load_yaml(fpath):
     """Return the content of a YAML file."""
     if not fpath.is_file():
-        emit.trace(f"Couldn't find config file {str(fpath)!r}")
+        emit.debug(f"Couldn't find config file {str(fpath)!r}")
         return
     try:
         with fpath.open("rb") as fh:
             content = yaml.safe_load(fh)
     except (yaml.error.YAMLError, OSError) as err:
-        emit.trace(f"Failed to read/parse config file {str(fpath)!r}: {err!r}")
+        emit.debug(f"Failed to read/parse config file {str(fpath)!r}: {err!r}")
         return
     return content
 
@@ -81,7 +84,7 @@ def get_templates_environment(templates_dir):
         # https://pyinstaller.readthedocs.io/en/stable/runtime-information.html
         # In this scenario we need to load from the data location that is unpacked
         # into the temporary directory at runtime (sys._MEIPASS).
-        emit.trace(f"Bundle directory: {sys._MEIPASS}")
+        emit.debug(f"Bundle directory: {sys._MEIPASS}")
         loader = FileSystemLoader(os.path.join(sys._MEIPASS, templates_dir))
     else:
         loader = PackageLoader("charmcraft", templates_dir)
@@ -177,7 +180,7 @@ def get_os_platform(filepath=pathlib.Path("/etc/os-release")):
             with filepath.open("rt", encoding="utf-8") as fh:
                 lines = fh.readlines()
         except FileNotFoundError:
-            emit.trace("Unable to locate 'os-release' file, using default values")
+            emit.debug("Unable to locate 'os-release' file, using default values")
         else:
             os_release = {}
             for line in lines:
@@ -227,3 +230,32 @@ def confirm_with_user(prompt, default=False) -> bool:
         return False
     else:
         return default
+
+
+def format_timestamp(dt: datetime.datetime) -> str:
+    """Convert a datetime object (with or without timezone) to a string.
+
+    The format is
+
+        <DATE>T<TIME>Z
+
+    Always in UTC.
+    """
+    # convert to UTC from whatever timezone `dt` has
+    dtz = datetime.datetime.fromtimestamp(time.mktime(dt.utctimetuple()))
+    return dtz.strftime("%Y-%m-%dT%H:%M:%SZ")
+
+
+def humanize_list(items: Iterable[str], conjunction: str) -> str:
+    """Format a list into a human-readable string.
+
+    :param items: list to humanize, must not be empty
+    :param conjunction: the conjunction used to join the final element to
+                        the rest of the list (e.g. 'and').
+    """
+    if not items:
+        raise ValueError("Cannot humanize an empty list.")
+    *initials, final = map(repr, sorted(items))
+    if not initials:
+        return final
+    return f"{', '.join(initials)} {conjunction} {final}"
